find_pokemon_stats:	; hl=pokemon id ; output : hl=pokemon stats
	ld a,(hl)

find_pokemon_stats_a:
	or a
	ret z
	ld hl,pokemon_stats
	ld bc,pokemon_stats_length-pokemon_stats

find_pokemon_stats_loop:
	add hl,bc
	dec a
	jp nz,find_pokemon_stats_loop
	or a
	sbc hl,bc
	ret

find_pokemon_data:	; input : hl=pokemon id ; output : hl=pokemon data
	ld a,(hl)

find_pokemon_data_a:
	ld b,a
	ld hl,pokemon_index

find_pokemon_data_loop:
	inc hl
	inc hl
	djnz find_pokemon_data_loop
	dec hl
	dec hl
	bcall _ldhlind
	ret

find_attack:	; input : hl=attack id ; output : hl=attack data
	ld a,(hl)

find_attack_a:
	ld b,a
	ld hl,attack_index
	jp find_pokemon_data_loop

find_pokemon_attack:	; input : hl=pokemon stats, c=attack id ; output : z:attack found, nz:attack not found, hl=attack data
	ld d,0
	ld e,pokemon_attacks-pokemon_stats
	add hl,de
	ld b,4

find_pokemon_attack_loop:
	ld a,(hl)
	cp c
	ret z
	inc hl
	inc hl
	inc hl
	djnz find_pokemon_attack_loop
	or $80	; reset zero flag
	ret

find_type:
	ld b,(hl)
	ld hl,type_index
	jp find_pokemon_data_loop

is_already_captured:	; input : a=id
	ld hl,pokemon_stats
	ld b,(pokemon_stats_end-pokemon_stats)/(pokemon_stats_length-pokemon_stats)

is_already_captured_loop:
	ld c,(hl)
	cp c
	ret z
	ld d,0
	ld e,pokemon_stats_length-pokemon_stats
	add hl,de
	djnz is_already_captured_loop
	or $80
	ret

generate_wild_pokemon:
	ld hl,map_pokemons
	ld b,(map_pokemons_end-map_pokemons)/4
	ld de,(mapAddr)

generate_wild_pokemon_find_map_data:	; on compare l'adresse de la carte avec celle de la liste des pokmons rencontrables sur chaque carte
	push hl
	bcall _ldhlind
	call cp_hl_de
	pop hl
	jr z,generate_wild_pokemon_find_map_data_ok
	inc hl	; premier mot (2 octet) = adresse de la carte
	inc hl	; /
	inc hl	; deuxime mot = adresse des pokmons de la carte
	inc hl	; /
	djnz generate_wild_pokemon_find_map_data
	ret	; pas de donnes concernant les pokmons rencontrables sur la carte donc pas de combat...

generate_wild_pokemon_find_map_data_ok:
	inc hl
	inc hl
	bcall _ldhlind	; on a les donnes des pokmons (ID et amplitudes des niveaux)
	ld a,(hl)	; nombre maximum de pokmons
	ld b,a
	call ionRandom
	ld d,0
	ld e,a	; on en choisit un au hasard (faudrait peut tre grer la raret ?)
	inc hl	; premier pokmon
	add hl,de	; ou plus
	add hl,de	;  |
	add hl,de	; /
	ld a,(hl)
	ld (opponent_pokemon_stats),a	; on a l'ID du pokmon, maintenant faut tirer un niveau au hasard
	push af
	inc hl
	push hl	; niveau min
	inc hl
	ld a,(hl)	; niveau max
	ld b,a
	call ionRandom
	pop hl
	ld b,a
	ld a,(hl)
	add a,b
	ld (opponent_pokemon_stats_lvl),a	; on a son niveau
	pop af	; maintenant on s'occupe de ses stats (PV, ATT et DEF), il faudra les augmenter en fonction du niveau (+courbe experience)
	call find_pokemon_data_a
	ld d,0
	ld e,(hl)
	add hl,de
	ld e,pokemon_hp_-pokemon_data
	add hl,de
	ld a,(opponent_pokemon_stats_lvl)
	push af
	push hl
	call calculate_hp	; on calcul ses pv en fonction de son niveau
	ld (opponent_pokemon_stats_hp),a
	ld (opponent_pokemon_stats_hp_max),a
	pop hl
	inc hl
	pop af
	push af
	push hl
	call calculate_stats	; idem pour ses stats d'attaque
	ld (opponent_pokemon_stats_att),a
	pop hl
	inc hl
	pop af
	push hl
	call calculate_stats	; idem pour ses stats de dfense
	pop hl
	ld (opponent_pokemon_stats_def),a
	xor a
	ld (opponent_pokemon_stats_state),a
	ld d,0
	ld e,pokemon_attacks_-pokemon_def_
	add hl,de	; on est aux attaques
	ld b,(hl)
	inc hl

generate_wild_pokemon_attacks:
	push bc
	ld a,(opponent_pokemon_stats_lvl)
	ld c,(hl)
	inc c
	sub c
	push af
	call p,generate_wild_pokemon_attacks_save
	pop af
	pop bc
	ret m
	inc hl
	inc hl
	djnz generate_wild_pokemon_attacks
	ret

generate_wild_pokemon_attacks_save:	; on sauvegarde les 4 meilleurs attaques (au maximum, selon le niveau)
	push hl
	pop de
	ld hl,opponent_pokemon_stats_attacks
	ld b,4	; faudra grer le "surnombrage"

generate_wild_pokemon_attacks_save_loop:
	ld a,(hl)
	or a
	jr z,generate_wild_pokemon_attacks_save_ok
	inc hl
	inc hl
	inc hl
	djnz generate_wild_pokemon_attacks_save_loop
	ret

generate_wild_pokemon_attacks_save_ok:
	ld a,(de)
	ld (hl),a
	ret

wait:	;	pause=480+300*b ns at 6MHz
	ei

waitloop:
	halt
	djnz wait
	di
	ret

cp_hl_de:
	or a
	sbc hl,de
	add hl,de
	ret

jp_hl:
	jp (hl)

_vputmap_onscreen:
	res textwrite,(IY+sgrflags)
	bcall _vputmap
	set textwrite,(IY+sgrflags)
	ret

dialogue:
	call _v_new_text
	jr _vputs_ml

_vdispa:
	ld h,0
	ld l,a
	bcall _setxxxxop2
	bcall _op2toop1
	ld a,3
	bcall _dispop1a
	ret

_vputs_ml:	;	_v_mode:1=text is typing; 0=text is directly drawn	|	string:$d6=new line; $07=pause and next text
	ld a,(pencol)
	push af
	push hl

_v_inc_loop:
	ld a,(hl)
	cp $d6
	jr z,_v_replace_zero
	cp 7
	jr z,_v_pause
	or a
	jp z,_v_disp_end
	inc hl
	jr _v_inc_loop

_v_replace_zero:
	ld (hl),0
	ex de,hl
	pop hl
	push de
_v_mode = $+1
	ld a,0
	or a
	bcallz _vputs
	call nz,_v_type
	pop de
	pop af
	push af
	push hl
	call _v_new_line
	ex de,hl
	ld (hl),$d6
	inc hl
	jr _v_inc_loop

_v_pause:
	ld (hl),0
	ex de,hl
	pop hl
	push de
	ld a,(_v_mode)
	or a
	bcallz _vputs
	call nz,_v_type
	push hl
	call _v_disp_pause
	pop hl
	pop de
	push hl
	xor a
	call _v_new_text
	ex de,hl
	ld (hl),7
	inc hl
	jr _v_inc_loop

_v_disp_pause:
_v_disp_no_pause = $+1
	ld a,0
	or a
	ret nz
	call _v_disp_pause_blink

_v_disp_pause_loop:
_v_disp_pause_count = $+1
	ld a,0
	inc a
	ld (_v_disp_pause_count),a
	cp scroll_cursor_speed
	call z,_v_disp_pause_blink
	ld b,1
	call wait
	ld a,$bf
	out (1),a
	in a,(1)
	bit 5,a
	ret z
	jr _v_disp_pause_loop

_v_disp_pause_blink:
	xor a
	ld (_v_disp_pause_count),a
	ld a,86
	ld l,54
	ld b,8
	ld ix,scroll_cursor
	call ionPutSprite
	jp ionFastCopy

_v_disp_end:
	pop hl
	pop af
	ld a,(_v_mode)
	or a
	bcallz _vputs
	push af
	call nz,_v_type
	pop af
	jr nz,_v_disp_pause
	ret

_v_new_line:
	ld (pencol),a
	ld a,(penrow)
	add a,6
	ld (penrow),a
	ret

_v_new_text:	;	a:0=frame on the bottom ; 1=frame on the top ; 2=battle frame
	push de
	push bc
	push hl
	push af
	ld a,2
	ld (pencol),a
	ld (penrow),a
	dec a
	ld (_v_mode),a
	pop af
	or a
	jr z,dialogue_down
	cp 1
	jr z,dialogue_up
	ld a,50
	ld (penrow),a
	ld bc,$600f
	ld de,$0031
	jr dialogue_end

dialogue_up
	ld bc,$1560
	ld de,0
	jr dialogue_end

dialogue_down:
	ld a,44
	ld (penrow),a
	ld bc,$1560
	ld de,$002b

dialogue_end:
	call draw_frame	; bcall _DrawRectBorder not available on 83
	call ionFastCopy
	pop hl
	pop bc
	pop de
	ret

_v_type:
	ld a,$df
	out (1),a
	in a,(1)
	bit 7,a
	jr z,_v_type_skip

	ld a,(hl)
	or a
	ret z
	inc hl
	cp 7
	jr z,_v_type
	cp $d6
	jr z,_v_type

	push bc
	ld b,5
	call wait
	pop bc

	bcall _vputmap
	push hl
	call ionFastCopy
	pop hl
	jr _v_type

_v_type_skip:
	bcall _vputs
	push hl
	call ionFastCopy
	pop hl
	ret

draw_frame:
	call rectangle_filled_solid
	inc d
	inc e
	dec b
	dec b
	dec c
	dec c
	jp rectangle_filled_xor

fade_out:
	ld a,(contrast)
	ld b,a

fade_out_loop:
	push bc
	ld a,b
	ld (contrast),a
	add a,$18
	or $c0
	bcall _lcdbusy
	out (10h),a
	ld b,fade_speed
	call wait
	pop bc
	djnz fade_out_loop
	ret

fade_in:
	ld hl,contrast
contrast_var = $+1
	ld a,0
	sub (hl)
	ld b,a

fade_in_loop:
	push bc
	inc (hl)
	ld a,(hl)
	add a,$18
	or $c0
	bcall _lcdbusy
	out (10h),a
	ld b,fade_speed
	call wait
	pop bc
	djnz fade_in_loop
	ret

flash_screen:
	push bc
	ld bc,$4060
	ld de,0
	push bc
	push de
	call rectangle_filled_xor
	call ionFastCopy
	pop de
	pop bc
	call rectangle_filled_xor
	call ionFastCopy
	pop bc
	djnz flash_screen
	ret

; Rectangle filled routine by chickendude
#ifdef TI83
GBUF_LSB = $29
GBUF_MSB = $8E
#endif
#ifdef TI83P
GBUF_LSB = $40
GBUF_MSB = $93
#endif

;b = height
;c = width
;d = starting x
;e = starting y
rectangle_filled_white:
	call rectangle_filled_solid

rectangle_filled_xor:
	ld a,$AE		;xor (hl)
	jr rectangle_filled2

rectangle_filled_solid:
	ld a,$B6		;or (hl)

rectangle_filled2:
	push de
	push bc
	ld (or_xor),a	;use smc for xor/solid fill
	ld a,d			;starting x
	and $7			;what bit do we start on?
	ex af,af'
	ld a,d			;starting x
	ld l,e			;ld hl,e
	ld h,0			; ..
	ld d,h			;set d = 0
	add hl,de		;starting y * 12
	add hl,de		;x3
	add hl,hl		;x6
	add hl,hl		;x12
	rra				;a = x coord / 8
	rra				;
	rra				;
	and %00011111	;starting x/8 (starting byte in gbuf)
	add a,GBUF_LSB
	ld e,a			;
	ld d,GBUF_MSB	;
	add hl,de		;hl = offset in gbuf
	ex af,af'
	or a			;clear carry and check if a = 0
	ld d,a
	ld a,%10000000
	jr z,$+6
	rra
	dec d
	jr nz,$-2
	ld e,12

rectangle_loop_x:
	push af
	push bc
	push hl
	ld c,a

rectangle_loop_y:
or_xor = $
	or (hl)			;smc will modify this to or/xor
	ld (hl),a
	ld a,c
	add hl,de
	djnz rectangle_loop_y
	pop hl
	pop bc
	pop af
	rrca
	jr nc,$+3
	inc hl
	dec c
	jr nz,rectangle_loop_x
rectangle_end:
	pop bc
	pop de
	ret

; Flip sprite routine by chickendude
; INPUT :
; ix:sprite; b:height; c:width/8
; OUTPUT :
; ix=flipped sprite
; destroys a, de, hl
; uses saferam3
flip_sprite:
	push bc
	push ix
	pop de
	ld hl,saferam3

flip_sprite_outer:
;de = original sprite
;hl = buffer
	push bc		;save b (sprite height)
	ld b,0		;bc = width in bytes
	add hl,bc	;go to very right of first sprite and draw leftward
	push hl
	ld b,8

flip_sprite_loop:
	dec hl		;go left in buffer
	push bc		;c gets overwritten so we need to save it
	ld a,(de)	;first byte in a
	rrca		;rotate a to the right and rotate the carry left into c
	rl c
	djnz $-3
	ld (hl),c	;save flipped byte
	inc de		;next byte of sprite
	pop bc		;c = counter (number of bytes to flip)
	dec c
	jr nz,flip_sprite_loop
	pop hl
	pop bc		;restore b (height)
	djnz flip_sprite_outer
	ld ix,saferam3
	pop bc
	ret